;----------------------------------------------------------------------------
;
; (C)Copyright 1989 Gibson Research, Incorporated
; All rights reserved.
;
; Creation date: 9/28/89
; Author . . . : Michael Toutonghi
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;
; This file contains general macros which can be used in many different
; programs.
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;
; Equates contained include:
;   TRUE		- NOT 0
;   FALSE		- 0
;   TYPE_CONST		- constant type using .TYPE directive
;   TYPE_REG		- register type using .TYPE directive
;
; Macros contained include:
;   MSDOS		- DOS access
;   BIOSVIDEO		- call BIOS interrupt 10H
;   BIOSKBD		- call BIOS interrupt 16H
;   DOSWRITE		- writes strings through DOS
;   STRINGINSET		- returns if a string is in a list of strings
;   MOUSECALL		- call to mouse driver
;   IODELAY		- delay to use between I/O instructions for fast CPUs
;   RESETINT		- resets the PIC with a non-specific EOI
;   OUTP		- flexible output macro
;   INP			- flexible input macro
;   MINVAL		- determine the minimum of 2 values (signed)
;   UMINVAL		- determine the minimum of 2 values (unsigned)
;   MAXVAL		- determine the maximum of 2 values (signed)
;   UMAXVAL		- determine the maximum of 2 values (unsigned)
;   GET_EGA_PARMS	- way to point to EGA/VGA parameters
;   STOREDWORD		- store a double word value to mem
;   GETDWORD		- get a double word value from memory
;   CHPOS		- point to first occurrence of char in a string
;   SKIPCHARS		- skip a set of chars in a string
;
;----------------------------------------------------------------------------

FALSE		EQU 0			;; define true and false
TRUE		EQU (NOT FALSE)		;; done
CR		EQU 13			;; carriage return
LF		EQU 10			;; line feed

TYPE_CONST	EQU (1 SHL 2)		;; constant type using .TYPE directive
TYPE_REG	EQU (1 SHL 4)		;; register type


; All DOS calls are made with this macro
MSDOS MACRO FuncNum
  IFB <FuncNum>
    INT 21H
  ELSE
    IF (FuncNum LT 256)
      MOV   AH,FuncNum
      INT 21H
    ELSE
      MOV   AX,FuncNum
      INT 21H
    ENDIF
  ENDIF
ENDM

; All BIOS video calls are made with this macro
BIOSVIDEO MACRO FuncNum
  IFB <FuncNum>
    INT 10H
  ELSE
    IF (FuncNum LT 256)
      MOV   AH,FuncNum
      INT 10H
    ELSE
      MOV   AX,FuncNum
      INT 10H
    ENDIF
  ENDIF
ENDM

; All BIOS keyboard calls are made with this macro
BIOSKBD MACRO FuncNum
  IFB <FuncNum>
    INT 16H
  ELSE
    IF (FuncNum LT 256)
      MOV   AH,FuncNum
      INT 16H
    ELSE
      MOV   AX,FuncNum
      INT 16H
    ENDIF
  ENDIF
ENDM

; Writes strings through DOS
; Parameters:
;   one or more addresses of strings in a list
; Returns:
;   all strings written through DOS
; Uses regs:
;   ax,dx
DOSWRITE MACRO STRINGS			;; writes series of strings through DOS
  IRP X,<STRINGS>
    lea     dx,X			;; point to string with dx
    MSDOS   9				;; write the string
  ENDM
ENDM

; Returns if a string is in a list of strings
; Parameters:
;   STR		- string to test
;   SET		- list of strings to check for (eg. <ax,bx,cx,...>)
;   MEMBERNUM	- assembler variable to determine the member number
; Returns:
;   MEMBERNUM = 0 if not found or the number (1 .. LastOne)
; Uses regs:
;   NONE
STRINGINSET MACRO STR,SET,MEMBERNUM
	LOCAL DECISIONVAR
	MEMBERNUM = 1			;; start w/1
	DECISIONVAR = 0			;; and no DECISIONVAR
  IRP X,<SET>
    IFIDNI <X>,<STR>
      DECISIONVAR = 1			;; exit with current number
    ENDIF
    IFE DECISIONVAR
      MEMBERNUM = (MEMBERNUM+1)		;; next member
    ENDIF
  ENDM
  IFE DECISIONVAR
    MEMBERNUM = 0			;; if it gets here, it failed
  ENDIF
ENDM

; This macro calls the mouse driver.
; Parameters:
;   M1,M2,M3,M4 - as in the mouse programmer's manual
; Returns:
;   as in the mouse programmer's ref
; Uses regs:
;   as in the mouse programmer's ref
MOUSECALL MACRO M1,M2,M3,M4
  IFNB <M1>
    IFDIFI <M1>,<ax>
      mov     ax,M1			; get first parm
    ENDIF
  ENDIF
  IFNB <M2>
    IFDIFI <M2>,<bx>
      mov	bx,M2			; get second parm
    ENDIF
  ENDIF
  IFNB <M3>
    IFDIFI <M3>,<cx>
      mov	cx,M3			; get third parm
    ENDIF
  ENDIF
  IFNB <M4>
    IFDIFI <M4>,<dx>
      mov	dx,M4			; get fourth parm
    ENDIF
  ENDIF
  int	33H				; call mouse driver
ENDM

; Just a simple delay for fast CPUs
IODELAY MACRO NumDelays
  IFB <NumDelays>
	jmp	SHORT $+2		;; delay for fast CPUs
  ELSE
    REPT <NumDelays>
	jmp	SHORT $+2		;; delay for fast CPUs
    ENDM
  ENDIF
ENDM

; Resets the interrupt controller with a non-specific EOI
RESETINT MACRO
	mov	al,20H			; get EOI
	out	20H,al			; out port
ENDM

; This macro just sends data out to either an indexed or
; non-indexed port.
; Parameters:
;   PortAdr   - address of port
;   PortVal   - value to output port
;   PortIdx   - index of port if it is an indexed port
;   NumDelays - number of delay jumps between outputs to port
; Returns:
;   al = index if indexed or value if not
;   ah = value if indexed or high word of value if word or untouched
;   dx = either port address or untouched if address is const < 255
;   value either output to port or output to indexed port
;   delays assembled as jump shorts
; Uses regs:
;   ax,dx
OUTP	MACRO PortAdr,PortVal,PortIdx,NumDelays
	Local DirectOut,WordOut,IndexOut
	DirectOut = 0		;; determines if we do an "OUT xx,AL"
	WordOut   = 0		;; determines if out ax or al
	IndexOut  = 0		;; determines if indexed (only for byte outs)

  IFNB <PortAdr>		;; if no port address, assume no parms
    IFDIFI <dx>,<PortAdr>	;; don't change it if it's in dx
      IFCONST PortAdr		;; see if const
	IF (PortAdr LT 255)	;; if port adr is less than 255 may be direct
	  IFB <PortIdx>		;; if no index, direct out
	    DirectOut = 1	;; set direct
	  ELSE
	    IFB <NumDelays>	;; if no delays, send word for index/data
	      DirectOut = 1	;; set direct
	    ENDIF
	  ENDIF
	ENDIF
      ENDIF
      IFE DirectOut
	mov	dx,PortAdr	;; port address is either in dx or direct out
      ENDIF
    ENDIF
    IFNB <PortVal>		;; check for port value > 256 or = ax
      IFCONST PortVal		;; see if const
	IF (PortVal LT 255)	;; if greater than word, word out
	  IFB <PortIdx>		;; if no index, put value in al
	    mov al,PortVal	;; done
	  ELSE
	    IndexOut = 1	;; indexed
	    IFIDNI <PortIdx>,<ah> ;; if already in ah
	      mov al,ah		;; get port index first
	      mov ah,PortVal	;; value in ah
	    ELSE
	      IFCONST PortIdx	;; is idx const
		mov ax,(PortVal SHL 8)+PortIdx ;; only need one move to ax
	      ELSE
		mov ah,PortVal	  ;; otherwise, value in ah
		IFDIFI <PortIdx>,<al> ;; port index should be in al
		  mov al,PortIdx  ;; put it there
		ENDIF
	      ENDIF
	    ENDIF
	    IFB <NumDelays>	;; if no delays, send word for index/data
	      WordOut = 1	;; word out
	    ENDIF
	  ENDIF
	ELSE
	  mov	ax,PortVal	;; get it in ax
	  WordOut = 1		;; set word out
	ENDIF
      ELSE			;; not const, check for ax or al
	IFIDNI <PortVal>,<ax>	;; if ax, word out
	  WordOut = 1		;; set
	ELSE
	  IFDIFI <PortVal>,<al> ;; if al, byte out, don't set al
	    IFB <PortIdx>	;; if no index, put value in al
	      mov al,PortVal	;; done
	    ELSE
	      IndexOut = 1	;; indexed
	      IFIDNI <PortIdx>,<ah> ;; if already in ah
		mov al,ah	;; get port index first
		mov ah,PortVal	;; value in ah
	      ELSE
		IFDIFI <PortVal>,<ah>
		  mov ah,PortVal ;; otherwise, value in ah
		ENDIF
		IFDIFI <PortIdx>,<al> ;; port index should be in al
		  mov al,PortIdx ;; put it there
		ENDIF
	      ENDIF
	      IFB <NumDelays>	;; if no delays, send word for index/data
		WordOut = 1	;; word out
	      ENDIF
	    ENDIF
	  ELSE
	    IFNB <PortIdx>	;; if index, put value in ah
	      IndexOut = 1	;; indexed
	      IFIDNI <PortIdx>,<ah> ;; if index already in ah
		xchg al,ah	;; trade al for ah
	      ELSE
		mov ah,al	;; otherwise, value in ah
		IFDIFI <PortIdx>,<al> ;; port index could still be al
		  mov al,PortIdx ;; if not, put it there
		ENDIF
	      ENDIF
	      IFB <NumDelays>	;; if no delays, send word for index/data
		WordOut = 1	;; word out
	      ENDIF
	    ENDIF
	  ENDIF
	ENDIF
      ENDIF
    ENDIF
  ENDIF
  IF DirectOut
    IF WordOut
	out	PortAdr,ax	;; done
    ELSE
      IF IndexOut
	out	PortAdr,al	;; send index
	IFNB <NumDelays>	;; delay if requested
	  IODELAY NumDelays
	ENDIF
	xchg	al,ah		;; index in ah, val in al
	out	PortAdr+1,al	;; send data
	xchg	al,ah		;; index in al, val in ah
      ELSE
	out	PortAdr,al	;; done
      ENDIF
    ENDIF
  ELSE
    IF WordOut
	out	dx,ax		;; done
    ELSE
      IF IndexOut
	out	dx,al		;; send index
	IFNB <NumDelays>	;; delay if requested
	  IODELAY NumDelays
	ENDIF
	xchg	al,ah		;; index in ah, val in al
	inc	dx		;; next port
	out	dx,al		;; send data
	xchg	al,ah		;; index in al, val in ah
	dec	dx		;; port back in dx
      ELSE
	out	dx,al		;; done
      ENDIF
    ENDIF
  ENDIF
  IFNB <NumDelays>		;; if delays requested, do them
    IODELAY NumDelays		;; done
  ENDIF
ENDM

; This reads from a VGA style indexed or non indexed port. In the
; interest of saving headache, eyestrain, and sanity, it is
; not as flexible as the OUTP macro.
; Parameters:
;   PortAdr - I/O address of port
;   PortIdx - index of port
; Returns:
;   ah = data read if indexed
;   al = index if indexed, data if not
;   dx = port address
; Uses regs:
;   ax,dx
INP	MACRO	PortAdr, PortIdx, NumDelays
  IFNB <PortAdr>
    IFDIFI <PortAdr>,<dx>
	  mov	  dx,PortAdr	;; get address in dx
    ENDIF
  ENDIF
  IFNB <PortIdx>
    IFDIFI <PortIdx>,<al>
	mov	al,PortIdx	;; index in al
    ENDIF
	out	dx,al		;; set index
    IFNB <NumDelays>		;; if delays requested, do them
	IODELAY NumDelays	;; done
	mov	ah,al		;; save index
	inc	dx		;; get data port
	in	al,dx		;; get data in al
	xchg	ah,al		;; value in ah, index in al
	dec	dx		;; port in dx
    ELSE
	in	ax,dx		;; read port
    ENDIF
    IFNB <NumDelays>		;; if delays requested, do them
	IODELAY NumDelays	;; done
    ENDIF
  ELSE
	in	al,dx		;; get port value in al
  ENDIF
ENDM

; Compares a destination variable with a source.
; If destination is less, it stays the same, otherwise a move
; instruction is generated to move the source to the destination.
; Signed compares are used. If the value is moved from source to
; destination and the optional label parameter is passed,
; control is transferred to the label.
MINVAL	MACRO Des,Src,StoreLabel
	Local	MacDone
	cmp	Des,Src			;; see if less
	jle	MacDone			;; if so, done
	mov	Des,Src			;; otherwise, get value from memory
	IFNB <StoreLabel>
	  jmp	SHORT StoreLabel	;; store value
	ENDIF
      MacDone:
ENDM

; Compares a destination variable with a source.
; If destination is less, it stays the same, otherwise a move
; instruction is generated to move the source to the destination.
; Unsigned compares are used. If the value is moved from source to
; destination and the optional label parameter is passed,
; control is transferred to the label.
UMINVAL MACRO Des,Src,StoreLabel
	Local	MacDone
	cmp	Des,Src			;; see if less
	jbe	MacDone			;; if so, done
	mov	Des,Src			;; otherwise, get value from memory
	IFNB <StoreLabel>
	  jmp	SHORT StoreLabel	;; store value
	ENDIF
      MacDone:
ENDM

; Compares a destination variable with a source.
; If destination is greater, it stays the same, otherwise a move
; instruction is generated to move the source to the destination.
; Signed compares are used. If the value is moved from source to
; destination and the optional label parameter is passed,
; control is transferred to the label.
MAXVAL	MACRO Des,Src,StoreLabel
	Local	MacDone
	cmp	Des,Src			;; see if greater
	jge	MacDone			;; if so, done
	mov	Des,Src			;; otherwise, get value from memory
	IFNB <StoreLabel>
	  jmp	SHORT StoreLabel	;; store value
	ENDIF
      MacDone:
ENDM

; Compares a destination variable with a source.
; If destination is greater, it stays the same, otherwise a move
; instruction is generated to move the source to the destination.
; Unsigned compares are used. If the value is moved from source to
; destination and the optional label parameter is passed,
; control is transferred to the label.
UMAXVAL MACRO Des,Src,StoreLabel
	Local	MacDone
	cmp	Des,Src			;; see if greater
	jae	MacDone			;; if so, done
	mov	Des,Src			;; otherwise, get value from memory
	IFNB <StoreLabel>
	  jmp	SHORT StoreLabel	;; store value
	ENDIF
      MacDone:
ENDM

; This macro gets the EGA parameters into a segment:offset
; register pair. The offset register must be able to be used
; as an index
; Parameters:
;   a data segment register
;   an offset register
; Returns:
;   the pair will contain a pointer to the video parameter table
; Uses regs:
;   the passed registers only
GET_EGA_PARMS	MACRO DSEG_REG,OFS_REG
	mov	OFS_REG,BIOS_DATA ;; first get the BIOS data segment in
	mov	DSEG_REG,OFS_REG ;; the segment register
	l&DSEG_REG OFS_REG,DSEG_REG:Bios_EGASave ;; get EGA video pointer
	l&DSEG_REG OFS_REG,DSEG_REG:[OFS_REG] ;; they now point to video parms
ENDM

; This macro stores a value from two registers into a location of memory.
STOREDWORD	MACRO MEM_LOC, SEG_VAL, OFS_VAL
	mov	word ptr MEM_LOC,OFS_VAL ;; put low word
	mov	word ptr MEM_LOC[2],SEG_VAL ;; high word
ENDM

; This macro reads a dword from memory
GETDWORD	MACRO SEG_VAL, OFS_VAL, MEM_LOC
	mov	OFS_VAL,word ptr MEM_LOC ;; put low word
	mov	SEG_VAL,word ptr MEM_LOC[2] ;; high word
ENDM

; DS must be in string segment.  Elements of CharSet will be
; compared directly against al.  A pointer to the char found
; will be in SI.
; Neither al or cx should be operands.	If MaxChars is used,
; carry is returned if no characters are found.  MaxChars is
; compared to the cx register so MaxChars must be a word
; value.
; Returns:
;   si = pointer to char found in string
;   cx = char number if maxlen is used
;   If not found then carry flag
CHPOS MACRO CharSet,String,MaxChars
  Local SetInStrEnd,SetInStrFound,SearchLoop,NotMax
    cld				 ; forward direction
    IFNB <String>
      IFDIF <String>,<[SI]>	 ; make sure we don't create a redundant stmnt
	IFDIF <String>,<[si]>	 ; make sure we don't create a redundant stmnt
	  lea	  si,String	 ; point to search string
	ENDIF
      ENDIF
    ENDIF

    IFNB <MaxChars>		 ; this part is done only if maxchars used
      xor   cx,cx		 ; start with 0 count
    ENDIF
  SearchLoop:
    IFNB <MaxChars>		 ; this part is done only if maxchars used
      inc   cx			 ; increment count
      cmp   cx,MaxChars		 ; see if chars are too high
      jbe   NotMax		 ; if OK, continue

      stc			 ; carry for not found
      jmp   SHORT SetInStrEnd	 ; done

     NotMax:
    ENDIF

    lodsb			 ; get char from string
    IRP X,<CharSet>		 ; compare every character
      cmp     al,X		 ; compare current character
      jz      SetInStrFound	 ; if found, done and carry is clear
    ENDM
    jmp     SHORT SearchLoop	 ; check next char

  SetInStrFound:
    dec     si			 ; point to char
    IFNB <MaxChars>		 ; this part is done only if maxchars used
      clc			 ; clear carry
    ENDIF

  SetInStrEnd:
ENDM

; DS must be in string segment.  Elements of CharSet will be
; compared directly against al.  On return, SI points to the
; first  char  not in the set.
; Neither al or cx should be operands.	If MaxChars is used,
; carry is returned if all characters are skipped.  MaxChars
; is compared to the cx register so it must be a word value.
SKIPCHARS MACRO CharSet,String,MaxChars
  Local SetInStrEnd,SearchLoop,NotMax
    cld				 ; forward direction
    IFNB <String>
      IFDIF <String>,<[SI]>	 ; make sure we don't create a redundant stmnt
	IFDIF <String>,<[si]>	 ; make sure we don't create a redundant stmnt
	  lea	  si,String	 ; point to search string
	ENDIF
      ENDIF
    ENDIF
    IFNB <MaxChars>		 ; this part is done only if maxchars used
      xor   cx,cx		 ; start with 0 count
    ENDIF
  SearchLoop:
    IFNB <MaxChars>		 ; this part is done only if maxchars used
      inc   cx			 ; increment count
      cmp   cx,MaxChars		 ; see if chars are too high
      jbe   NotMax		 ; if OK, continue

      stc			 ; carry for not found
      jmp   SHORT SetInStrEnd	 ; done

     NotMax:
    ENDIF

    lodsb			 ; get char from string
    IRP X,<CharSet>		 ; compare every character
      cmp     al,X		 ; compare current character
      jz      SearchLoop	 ; if in set, skip it
    ENDM
    dec     si			 ; point to char

    IFNB <MaxChars>		 ; this part is done only if maxchars used
      clc			 ; character not in set is found, clear carry
    ENDIF

  SetInStrEnd:
ENDM

