;----------------------------------------------------------------------------
;
; (C)Copyright 1989 Gibson Research, Incorporated
; All rights reserved.
;
; Creation date: 10/02/89
; Author . . . : Michael Toutonghi
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;
; This file contains segment definitions and equates for accessing BIOS and
; DOS system variables.
;
;----------------------------------------------------------------------------

; Equipment flag bit masks and flags
EQUIP_VID_MSK	EQU 30H			; mask for installed video equipment
EQUIP_MONO	EQU 30H			; monochrome board default
EQUIP_BW80	EQU 20H			; default color 80 column
EQUIP_BW40	EQU 10H			; default color 40 column

; Bios_Info bit flags and masks
EGA_ACTIVE_BIT	EQU 8			; clear when EGA/VGA is active display
BCURSOR_EM_MSK	EQU 1			; set to 1 for no emulation

; Base video segments
MONO_BASE_SEG	EQU 0B000H		; mono standard
CGA_BASE_SEG	EQU 0B800H		; color standard
EGA_BASE_SEG	EQU 0A000H		; extended mode EGA/VGA standard

; Base EGA/VGA ports
EGA_MISCOUT_R	EQU 3CCH		; read misc out reg
EGA_MISCOUT_W	EQU 3C2H		; write misc out reg
EGA_GDC		EQU 3CEH		; graphics display controller
EGA_SEQ		EQU 3C4H		; timing sequencer
CRTC_COLOR	EQU 3D4H		; color crt controller
CRTC_MONO	EQU 3B4H		; monochrome crt controller

; Video parameter table entry
VIDParmEntry	STRUC
  VParmCols	db ?			; columns for this mode
  VParmRows	db ?			; rows for this mode
  VParmByteChar db ?			; font bytes per character
  VParmModeSize dw ?			; size of this mode in words
  VParmSeqRegs	db 4 dup(?)		; sequencer parameters
  VParmMiscReg	db ?			; miscellaneous out register
  VParmCRTRegs	db 19h dup(?)		; CRT controller registers
  VParmAttRegs	db 14h dup(?)		; attribute controller registers
  VParmGDCRegs	db 9 dup(?)		; graphics controller registers
VIDParmEntry	ENDS

; This structure defines the area pointed to by the EGA/VGA
; save pointer
EGASaveArea	STRUC
  EGAVidParmPtr dd ?			; EGA/VGA video parameters
  EGADynSave	dd ?			; dynamic save area for pallette regs
  EGAResPtrs	dd 6 dup(0)		; reserved pointers
EGASaveArea	ENDS

; standard BIOS data segment
BIOS_DATA SEGMENT AT 40H

	ORG 10H
	Equip_Flag	db ?		; The equipment flag

	; Video BIOS variables
	ORG 49H
	Bios_CRT_Mode	db ?		; the BIOS mode variable
	Bios_CRT_Cols	dw ?		; BIOS columns var
	Bios_CRT_Len	dw ?		; the BIOS CRT len variable
	Bios_CRT_Start	dw ?		; page offset address
	Bios_CPosArr	dw 8 dup(?)	; cursor array
	Bios_CurAttr	dw ?		; cursor attributes
	Bios_ActPage	db ?		; visible page
	Bios_6845	dw ?		; CRT controller base
	Bios_ModeSet	db ?		; CGA 3x8 register
	Bios_Pallette	db ?		; 3x9 pallette register

	; Timer BIOS variables
	ORG 6CH
	Bios_TimerLow	dw ?		; timer low word
	Bios_TimerHigh	dw ?		; timer high

	; EGA/VGA specific BIOS variables
	ORG 84H
	Bios_CRows	db ?		; number of rows on screen -1
	Bios_Points	dw ?		; bytes per character
	Bios_Info	db ?		; info bits


	ORG 0A8H
	Bios_EGASave	dd ?		; BIOS EGA save pointer

BIOS_DATA ENDS

PSP_SEG SEGMENT AT 0
	ORG 2CH
	PSPEnvironSeg	dw ?		; environment seg
	ORG 80H
	PSPNumParmBytes db ?		; number of bytes in the parameter
	PSPComParm	db 127 dup(?)	; command line parameter
PSP_SEG ENDS

