NAME	TEST
TITLE	Very Simple Test For Object Inheritance

;----------------------------------------------------------------------------
;
; (C)Copyright 1989 Gibson Research, Incorporated
;
; Creation date: 10/02/89
;
; Author . . . : Michael Toutonghi
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;
; This program simply provides a code fragment which illustrates object
; oriented assembly language programming.
;
;----------------------------------------------------------------------------

INCLUDE COMSEGS.INC
INCLUDE STDMACS.INC
INCLUDE OBJECT.INC

;----------------------------------------------------------------------------
;
; We use the INITEXIT object to demonstrate how object modules can
; automatically be initialized.
;
;----------------------------------------------------------------------------
USE_OBJECTS <INITEXIT>

; This is a simple 2D point definition with it's own methods
; for translating itself
OBJECT		Point
  ODW XCoord	?			; Define the X and Y coordinates
  ODW YCoord	?

  INIT_EXIT	InitAll,Exit		; define initialization and exit method

  METHOD	Move,Virtual		; Define the methods
OBJECT_END

; This object inherits data from the 2D point and adds a Z
; coordinate to make it 3D
OBJECT		Point3D,Point
  ZCoord	dw ?			; the additional coordinate

  METHOD	Move,Virtual		; new methods for movement
OBJECT_END

_DATA	SEGMENT

	MyPoint Point3D <>		; define one 3D point as a test

_DATA	ENDS

_TEXT	SEGMENT
	ASSUME_CS
	ORG 100H			; setup for .COM file

; This is the main procedure of the test program.
; Parameters:
;   es	  = PSP
;   ds	  = PSP
; Returns:
;   terminates normally
; Uses regs:
;   all
_main		proc near

	InitExit_Init			; initialize all INIT_EXIT methods

	Point3D_Move MyPoint,20,15,-99	; translate the point

	xor	ax,ax			; exit code of 0, normal termination
	InitExit_Exit			; exit

_main		endp

; This is the initialization method which can be used to
; setup interrupt vectors, etc. This particular method has
; no necessary function and is included as an illustration.
; Parameters:
;   es	  = DGROUP
;   ds	  = DGROUP
; Returns:
;   nothing
; Uses regs:
;   can use ax,bx,cx,dx,si
_Point_InitAll proc near

	ret				; return

_Point_InitAll endp

; This is the exit method which can be used to
; release interrupt vectors, etc. This particular method has
; no necessary function and is included as an illustration.
; Parameters:
;   es	  = DGROUP
;   ds	  = DGROUP
; Returns:
;   nothing
; Uses regs:
;   can use ax,bx,cx,dx,si
_Point_Exit proc near

	ret				; return

_Point_Exit endp

; This method simply translates the current point object
; Parameters:
;   ds:di = pointer to point data structure
;   ax	  = X value for translate
;   bx	  = Y value for translate
; Returns:
;   data which makes the point moved in 2 space
; Uses regs:
;   NONE
_Point_Move	proc near

	add	[di].XCoord,ax		; translate X
	add	[di].YCoord,bx		; translate Y
	ret				; return

_Point_Move	endp

; This routine simply translates the current point object
; Parameters:
;   ax = X value for translate
;   bx = Y value for translate
;   cx = Z value for translate
;   es:di = pointer to point data structure
; Returns:
;   data which makes the 3D point moved in 3 space
; Uses regs:
;   NONE
_Point3D_Move	proc near

	add	[di].ZCoord,cx		; translate along Z axis
	Point3D_Move_A [di]		; do ancestor's move
	ret				; done

_Point3D_Move	endp

_TEXT	ENDS
END	_main
